import spacy
from gensim.corpora import Dictionary, MmCorpus
from gensim import models

print('Loading models...')

nlp = spacy.load('en_core_web_lg')

try:
    dictionary = Dictionary.load('dictionary')
    corpus = MmCorpus('corpus')
except Exception as e:
    print(e)

    print('Loading source data...')

    texts = []
    with open('ap.txt') as input_file:
        texts = input_file.readlines()

    print('Generating documents...')

    documents = [[t.lemma_ for t in nlp(text) if not t.is_stop and t.pos_ == 'NOUN'] for text in texts]

    print('Building dictionary...')

    dictionary = Dictionary()
    dictionary.add_documents(documents)

    print('Postprocessing dictionary...')

    dictionary.filter_extremes()
    dictionary.compactify()
    dictionary.save('dictionary')

    print('Building corpus...')

    corpus = [dictionary.doc2bow(doc) for doc in documents]
    MmCorpus.serialize('corpus', corpus, id2word=dictionary)

print('Building models...')

mdl = models.LsiModel(corpus, id2word=dictionary, num_topics=50)
print('LSI Topics')
for topic in mdl.print_topics():
    print(topic)

mdl = models.LdaModel(corpus, id2word=dictionary, num_topics=50)
print('LDA Topics')
for topic in mdl.print_topics():
    print(topic)
